<?php

namespace Drupal\mcapi_cc\CreditCommons;

use CreditCommons\Entry;
use Drupal\mcapi_cc\CreditCommons\Account;
use Drupal\mcapi_cc\CreditCommons\AccountRemote;
use Drupal\mcapi\Entity\Transaction as McapiTransaction;

class Transaction extends \CreditCommons\BaseTransaction implements \CreditCommons\TransactionInterface {

  static function createFromMcapi(McapiTransaction $transaction, $wallet_id) {
    $remote_path = $transaction->entries->first()->metadata[$wallet_id]??'';
    foreach ($transaction->entries as $ent) {
      if ($ent->payee_wid == $wallet_id) {
        $payee = new AccountRemote($remote_path);
        $payer = new Account($ent->payer_wid);
      }
      elseif ($ent->payer_wid == $wallet_id) {
        $payer = new AccountRemote($remote_path);
        $payee = new Account($ent->payee_wid);
      }
      // Multiply the quant because this transaction is about to be sent trunkwards.
      $entries[] = new Entry(
        $payee,
        $payer,
        $ent->quant * \Drupal::config('mcapi_cc.settings')->get('trunkward.rate'),
        (object)$ent->metadata,
        $ent->description
      );
    }

    return new static (
      uuid: $transaction->uuid(),
      type: $transaction->workflow->target_id,
      state: $transaction->state->value,
      entries: $entries,
      written: $transaction->created->value,
      version: 0
    );
  }

  function getWorkflow() : \CreditCommons\Workflow {
    // this is never called.
  }

}