<?php

namespace Drupal\mcapi_cc\Plugin\Derivative;

use Drupal\mcapi\Entity\Workflow;
use Drupal\Core\Plugin\Discovery\ContainerDeriverInterface;
use Drupal\Component\Plugin\Derivative\DeriverBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides local tasks to the transact/ page for remote-only workflows
 * @todo there's a bug which prevents the 'Remote' tab from being active with non-default sub tabs.
 */
class TransactionFormTasks extends DeriverBase implements ContainerDeriverInterface {

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container, $base_plugin_id) {
    return new static();
  }

  /**
   * {@inheritDoc}
   * Make a default tab for remote transaction forms
   */
  public function getDerivativeDefinitions($base_plugin_definition) {
    $home = \Drupal::config('mcapi_cc.settings')->get('node_name');
    $mcTypes = \Drupal::entityQuery('mc_workflow')->accessCheck()
      ->condition('home', $home, '<>')
      ->condition('disabled', 0)
      ->execute();
    $w = 0;
    // In Clearingcentral mode the builtin transaction forms don't work because
    // we can't use the same wallet picker widget for local and remote
    // transactions. So we have to provide separate forms, under their own
    // remote tab with their own base_route.
    foreach (Workflow::loadMultiple($mcTypes) as $workflow) {
      if ($workflow->isLocal())continue;
      /** @var \CreditCommons\Workflow $workflow */
      $id = 'remote.'.$workflow->id;
      $this->derivatives[$id] = [
        'title' => $workflow->label,
        'description' => $workflow->summary,
        'route_name' => 'entity.mc_transaction.add_remote',
        'route_parameters' => [
          'mc_workflow' => $workflow->id
        ],
        'base_route' => 'entity.mc_transaction.add_remote',
        'parent_id' => 'mcapi_cc.autotasks:form.remote',
        'weight' => $w++,
      ] + $base_plugin_definition;
    }
    if (count($this->derivatives) > 1) {
      // Move all under the new 'Remote' tab.
      foreach ($this->derivatives as $task) {
        unset($task['base_route']);
        $task['parent_id'] == 'mcapi_cc.autotasks:form.remote';
      }
      // Create the remote tab.
      $first = reset($this->derivatives);
      $this->derivatives['form.remote'] = [
        'title' => t('Remote'),
        'description' => $first['description'],
        'route_name' => $first['route_name'],
        'base_route' => 'entity.mc_transaction.add_form',
        'route_parameters' => $first['route_parameters'],
        'id' => 'form.remote',
        'options' => [],
        'weight' => 10, // Aim for the last tab.
      ] + $base_plugin_definition;
    }
    return $this->derivatives;
  }

}
