<?php

namespace Drupal\mcapi\Form;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\mcapi\Entity\Workflow;
use CreditCommons\TransactionInterface as CCTransactionInterface;

/**
 * Build form to edit, workflow config entity
 * @todo finish the complicated bits.
 */
class WorkflowEdit extends EntityForm {

  protected $allStates;

  function init(FormStateInterface $form_state) {
    parent::init($form_state);
    $this->allStates = mc_states(TRUE);
  }

  /**
   * {@inheritDoc}
   */
  public function getFormId() {
    return 'mcapi_workflow_form';
  }

  /**
   * {@inheritDoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $w = 0;
    $form = parent::form($form, $form_state);
    $form['#title'] = $this->entity->isNew() ?
      $this->t('New transaction workflow') :
      $this->t("Editing '@name' workflow", ['@name' => $this->entity->label()]);

    $form['label'] = [
      '#title' => $this->t('Workflow name'),
      '#type' => 'textfield',
      '#default_value' => $this->entity->label(),
      '#size' => 40,
      '#maxlength' => 80,
      '#required' => TRUE,
      '#weight' => $w++
    ];
    $form['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $this->entity->id(),
      '#maxlength' => 12,
      '#machine_name' => [
        'exists' => '\Drupal\mcapi\Entity\Workflow::load',
      ],
      '#disabled' => !$this->entity->isNew(),
    ];
    $form['summary'] = [
      '#title' => $this->t('Summary'),
      '#type' => 'textfield',
      '#default_value' => $this->entity->summary,
      '#size' => 40,
      '#maxlength' => 80,
      '#required' => TRUE,
      '#weight' => $w++
    ];

    $form['creation'] = [
      '#title' => $this->t('Creating the transaction'),
      '#type' => 'fieldset',
      '#tree' => TRUE,
      '#weight' => $w++,
      'state' => [
        '#title' => t('Initial state'),
        '#type' => 'select',
        '#empty_option' => '-'.$this->t('Select').'-',
        '#required' => TRUE,
        '#options' =>  $this->allStates,
        '#default_value' => $this->entity->creation ? $this->entity->creation['state'] : CCTransactionInterface::STATE_COMPLETED,
        '#ajax' => [
          'callback' => [$this, 'showStatesFieldsOnly'],
          'wrapper' => 'edit-states',
        ]
      ],
      'confirm' => [
        '#title' => t('Confirm transaction on a separate page'),
        '#type' => 'checkbox',
        '#default_value' => $this->entity->creation ? $this->entity->creation['confirm'] : FALSE
      ]
    ];

    $form['direction'] = [
      '#title' => $this->t('Direction'),
      '#type' => 'radios',
      '#options' => [
        Workflow::THIRDPARTY => $this->t('3rdparty (usually admin only)'),
        Workflow::BILL => $this->t('Bill - the user is asking for a payment.'),
        Workflow::CREDIT => $this->t('Credit - the user is sending a payment.'),
      ],
      '#default_value' => $this->entity->direction,
      '#required' => TRUE,
      '#weight' => $w++
    ];
    $form['states'] = [
      '#markup' => '<div id="edit-states"></div>',
      '#weight' => $w++
    ];
    if (isset($this->entity->creation['state'])) {
      $form['states'] = $this->showStatesFields($form, $form_state);
      $form['states']['#weight'] = $w++;
    }
    $form['state'] = [
      '#type' => 'checkbox',
      '#title' => t('Disabled'),
      '#default_value' => $this->entity->disabled,
      '#weight' => $w++
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  function validateForm(array &$form, FormStateInterface $form_state) {
    // No validation required.
  }

  /**
   * {@inheritDoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    parent::save($form, $form_state);
    $form_state->setRedirect('entity.mc_workflow.collection');
  }

  /**
   * {@inheritdoc}
   */
  public function buildEntity(array $form, FormStateInterface $form_state) {
    $entity = parent::buildEntity($form, $form_state);
    unset($entity->states['new_transition']);
    foreach ($entity->states as $from => $to) {
      foreach ($to as $dest => $info) {
        unset($info['del']);
      }
    }
    $trigger_element = $form_state->getTriggeringElement();
    if ($form_state->getValue(['states', 'new_transition'])) {
      [$from_state, $dest_state] = explode('-', $form_state->getValue(['states', 'new_transition']));
      $entity->states[$from_state][$dest_state] = ['actor' => '', 'label' => ''];
    }
    elseif ($trigger_element and substr($trigger_element['#name'], 0, 3) == 'del') {
      [,$source, $dest] = explode('-', $trigger_element['#name']);
      unset($entity->states[$source][$dest]);
    }
    return $entity;
  }

  /**
   * {@inheritdoc}
   */
  protected function actions(array $form, FormStateInterface $form_state) {
    $actions = parent::actions($form, $form_state);
    if ($this->entity->isLocal() and !$this->entity->isNew()) {
      $actions['delete']['#url']->setOption('query', ['destination' => '/admin/accounting']);
    }
    else {
      unset($actions['delete']);
    }
    return $actions;
  }


  /**
   * Generate the nested states part of the form based on the entity.
   */
  function showStatesFields($form, $form_state) {
    $additional_fields = [
      '#title' => $this->t('Transitions'),
      '#type' => 'details',
      '#open' => TRUE,
      '#tree' => TRUE,
      '#description' => $this->t('Admin can do all defined transitions.'),
    ];
    foreach ($this->entity->states as $from_state => $to_states) {
      if (empty($to_states)) continue;
      $additional_fields[$from_state] = [
        '#title' => $this->allStates[$from_state],
        '#type' => 'details',
        '#collapsed' => FALSE
      ];
      foreach ($to_states as $to_state => $info) {
        $additional_fields[$from_state][$to_state] = [
          '#title' => $this->allStates[$to_state],
          '#type' => 'details',
          '#open' => TRUE,
          'label' => [
            '#title' => $this->t('Name'),
            '#type' => 'textfield',
            '#required' => TRUE,
            '#default_value' => $info['label'],
            '#weight' => 1
          ],
          'actor' => [
            '#title' => $this->t('Actor'),
            '#description' => $this->t('Admin can always do this'),
            '#type' => 'checkboxes',
            '#options' => [
              'payee' => $this->t('Payee'),
              'payer' => $this->t('Payer')
            ],
            '#required' => FALSE,
            '#empty_option' => $this->t('Admin only'),
            '#default_value' => $info['actor'],
            '#weight' => 2
          ],
          'del' => [
            '#type' => 'button',
            '#name' => 'del-'.$from_state.'-'.$to_state,
            '#value' => $this->t("Delete '@transition' transition", ['@transition' => $info['label']]),
            #'#url' => Url::fromUserInput('/'),
            '#ajax' => [
              'callback' => [$this, 'removeTransition'],
              'wrapper' => 'edit-states',
            ],
            '#weight' => 5
          ]
        ];
      }
    }
    if ($options = $this->unusedTransitions()) {
      // @todo. Bad selection when making new type.
      $additional_fields['new_transition'] = [
        '#title' => $this->t('Add transition'),
        '#type' => 'select',
        '#empty_option' => ' - '.$this->t('Select').' - ',
        '#default_value' => '',
        '#options' => $options,
        '#ajax' => [
          'callback' => [$this, 'showStatesFieldsOnly'],
          'wrapper' => 'edit-states',
        ]
      ];
    }
    return $additional_fields;
  }

  /**
   * Ajax callback
   * @param type $form
   * @param type $form_state
   * @return array
   */
  function showStatesFieldsOnly($form, $form_state) {
    return $form['states'];
  }

  function removeTransition($form, $form_state) {
    return $form['states'];
  }

  /**
   * Get a list of possible, but unused transitions.
   * @return array
   */
  function unusedTransitions() : array {
    // Build a list of all possible transitions, excluding the ones already used.
    $transitions = [];
    if (empty($this->entity->creation['state'])) {
      // Should never happen
    }
    if (empty($this->entity->states)) {
      // There are no transitions after the transaction is created.
      $transitions[$this->entity->creation['state']] = $this->removeState($this->entity->creation['state']);
    }
    else {
      foreach (array_keys($this->allStates) as $from_state) {
        foreach (array_keys($this->allStates) as $dest_state) {
          if ($from_state == $dest_state) continue;
          if (isset($this->entity->states[$from_state][$dest_state])) continue;
          $transitions[$from_state][] = $dest_state;
        }
      }
    }
    $options = [];
    foreach ($transitions as $start_state => $end_states) {
      foreach ($end_states as $end_state => $info) {
        $options["$start_state-$end_state"] =  t(
          '@start_state -> @end_state',
          ['@start_state' => $this->allStates[$start_state], '@end_state' => $this->allStates[$end_state]]
        );
      }
    }
    return $options;
  }

  /**
   * Generate the part of the form which chooses a new dest state given a new from state
   */
  function addTransition($form, $form_state) {
    [$from_state, $dest_state] = explode('-', $form_state->getValue('states')['new_transition']);
    $form['states'][$from_state][$dest_state] = [
      '#title' => $dest_state,
      '#type' => 'details',
      '#weight' => 7,
      'to_state' => [
        '#title' => $this->t("Add 'dest' state"),
        '#type' => 'select',
        '#required' => TRUE,
        '#empty_option' => '-'.$this->t('Select').'-',
        '#options' => $choices,
        '#ajax' => [
          'callback' => [$this, 'addDestState'],
          'wrapper' => "edit-states",
        ]
      ]
    ];
    return $form['states'];
  }

  /**
   * Remove a state from a given list of states.
   * @param string $states
   * @param string $state
   * @return array
   */
  protected function removeState(string $state) : array {
    $states = $this->allStates;
    foreach (func_get_args() as $state) {
      unset($states[$state]);
    }
    return $states;
  }

}

