<?php

namespace Drupal\mcapi\Plugin\Field\FieldType;

use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\TypedData\DataReferenceTargetDefinition;
use Drupal\Core\Form\FormStateInterface;

/**
 * Plugin implementation of the 'worth' field type.
 *
 * @FieldType(
 *   id = "mc_worth",
 *   label = @Translation("Worth"),
 *   description = @Translation("A quantity of community currency."),
 *   category = "Number",
 *   default_formatter = "mc_worth",
 *   default_widget = "mc_worth_nullable",
 * )
 */
class WorthItem extends FieldItemBase {

  /**
   * {@inheritDoc}
   */
  public static function propertydefinitions(FieldStorageDefinitionInterface $field_definition) {
    $properties['quant'] = DataReferenceTargetDefinition::create('integer')
      ->setLabel(new TranslatableMarkup('Worth'))
      ->setDescription(new TranslatableMarkup('A quantity of community currency.'))
      ->setRequired(FALSE);
    // consider adding a computer field for the whole user.
    return $properties;
  }

  /**
   * {@inheritDoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition) {
    return [
      'columns' => [
        'value' => [
          'description' => 'units of community currency',
          'type' => 'int',
          'size' => 'normal',
          'not null' => FALSE,
        ],
      ]
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultFieldSettings() {
    return [
      'minus' => TRUE
    ] + parent::defaultFieldSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function fieldSettingsForm(array $form, FormStateInterface $form_state) {
    return $form + [
      'minus' => [
        '#type' => 'checkbox',
        '#title' => $this->t('Format as a negative value'),
        '#default_value' => $this->getSetting('minus')
      ]
    ];
  }


  /**
   * {@inheritDoc}
   * @note not used r/n because balance of trade wallet is 0
   */
  public function __isEmpty() {
    return is_null($this->quant);
  }

  /**
   * {@inheritDoc}
   */
  public static function generateSampleValue(FieldDefinitionInterface $field_definition) {
    if (rand(0, 9) > 7) {
      return rand(1000, 5000);
    }
    else {
      return NULL;
    }
  }

}
