<?php

namespace Drupal\mcapi\Plugin\Field;

use Drupal\Core\Field\FieldItemList;

/**
 * A signature field list
 */
class SignatureFieldItemList extends FieldItemList {

  /**
   * Append the required signature to the transaction.
   *
   * @param int $uid
   * @return void
   * @throws \Exception
   *
   * @note Does NOT save the transaction.
   */
  function sign() : void {
    $values = $this->getValue();
    foreach ($values as &$val) {// should be only 1
      $val['signed'] = \Drupal::time()->getRequestTime();
      $this->setValue($values);
    }
  }

  /**
   * Get the uids of the users who have yet to sign.
   * @return array
   * @todo this is a bit superfluous now that transactions can have only one signatory.
   */
  function getUnsigned() : array {
    $unsigned = [];
    foreach ($this->getValue() as $val) {
      if (empty($val['signed'])) {
        $unsigned[] = $val['uid'];
      }
    }
    return $unsigned;
  }

  /**
   *
   * @todo this is a bit superfluous now that transactions can have only one signatory.
   */
  function hasSigned(int $uid) : bool {
    foreach ($this->getValue() as $val) {
      if ($val['uid'] == $uid and !empty($val['signed'])) {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
   *
   * @todo this is a bit superfluous now that transactions can have only one signatory.
   */
  function removeSignature($uid) {
    foreach ($this->getValue() as $delta => $val) {
      if ($val['uid'] == $uid) {
        $this->removeItem($delta);
      }
    }
  }

  /**
   *
   */
  public function appendItem($value = NULL) {
    foreach ($this->list as $item) {
      if ($item->uid == $value['uid']) {
        return;
      }
    }
    return parent::appendItem($value);
  }

}
